#include "ray.h"

double Ray::env_ior = 1.0;

Ray::Ray() {
	ior = env_ior;
	energy = 1.0;
	time = 0.0;
}

Ray::Ray(const Vector3 &origin, const Vector3 &dir) {
	this->origin = origin;
	this->dir = dir;
	ior = env_ior;
	energy = 1.0;
	time = 0.0;
}

void Ray::transform(const Matrix4x4 &xform) {
	Matrix4x4 inv_transp_xform = xform.inverse();
	inv_transp_xform.transpose();
	
	origin.transform(xform);
	dir.transform(inv_transp_xform);
}

Ray Ray::transformed(const Matrix4x4 &xform) const {
	Matrix4x4 inv_transp_xform = xform.inverse();
	inv_transp_xform.transpose();
	
	return Ray(origin.transformed(xform), dir.transformed(inv_transp_xform));
}

void Ray::enter(double new_ior) {
	ior = new_ior;
	ior_stack.push(ior);
}

void Ray::leave() {
	ior_stack.pop();
	ior = ior_stack.empty() ? env_ior : ior_stack.top();
}
